]> git.sesse.net Git - cubemap/blobdiff - server.h
Support parsing streams from config file. Also support multiple streams (includes...
[cubemap] / server.h
index 6311c68095c6ef9558bbb7518fb5e8d9a2caae02..cc1b78dd2a6a95da7fd7e4ec9819714c2170518c 100644 (file)
--- a/server.h
+++ b/server.h
@@ -3,23 +3,32 @@
 
 #include <stdint.h>
 #include <pthread.h>
+#include <sys/epoll.h>
 #include <string>
 #include <map>
+#include <vector>
 
-#define NUM_SERVERS 4
 #define BACKLOG_SIZE 1048576
 #define EPOLL_MAX_EVENTS 8192
 #define EPOLL_TIMEOUT_MS 20
 #define MAX_CLIENT_REQUEST 16384
 
+class ClientProto;
+class CubemapStateProto;
+class StreamProto;
+
 struct Client {
        Client() {}
        Client(int sock);
 
+       // Serialization/deserialization.
+       Client(const ClientProto &serialized);
+       ClientProto serialize() const;
+
        // The file descriptor associated with this socket.
        int sock;
 
-       enum State { READING_REQUEST, SENDING_HEADER, SENDING_DATA };
+       enum State { READING_REQUEST, SENDING_HEADER, SENDING_DATA, SENDING_ERROR };
        State state;
 
        // The HTTP request, as sent by the client. If we are in READING_REQUEST,
@@ -32,31 +41,45 @@ struct Client {
 
        // The header we want to send. This is nominally a copy of Stream::header,
        // but since that might change on reconnects etc., we keep a local copy here.
-       // Only relevant for SENDING_HEADER; blank otherwise.
-       std::string header;
+       // Only relevant for SENDING_HEADER or SENDING_ERROR; blank otherwise.
+       std::string header_or_error;
 
-       // Number of bytes we've sent of the header. Only relevant for SENDING_HEADER.
-       size_t header_bytes_sent;
+       // Number of bytes we've sent of the header. Only relevant for SENDING_HEADER
+       // or SENDING_ERROR.
+       size_t header_or_error_bytes_sent;
 
        // Number of bytes we've sent of data. Only relevant for SENDING_DATA.
        size_t bytes_sent;
 };
 
 struct Stream {
+       Stream(const std::string &stream_id);
+       ~Stream();
+
+       // Serialization/deserialization.
+       Stream(const StreamProto &serialized);
+       StreamProto serialize() const;
+
+       std::string stream_id;
+
        // The HTTP response header, plus the video stream header (if any).
        std::string header;
 
        // The stream data itself, stored in a circular buffer.
-       char data[BACKLOG_SIZE];
+       char *data;
 
        // How many bytes <data> contains. Can very well be larger than BACKLOG_SIZE,
        // since the buffer wraps.
        size_t data_size;
+
+private:
+       Stream(const Stream& other);
 };
 
 class Server {
 public:
        Server();
+       ~Server();
 
        // Start a new thread that handles clients.
        void run();
@@ -64,8 +87,14 @@ public:
        // Stop the thread.
        void stop();
 
+       CubemapStateProto serialize() const;
+
        void add_client(int sock);
+       void add_client_from_serialized(const ClientProto &client);
+
        void add_stream(const std::string &stream_id);
+       void add_stream_from_serialized(const StreamProto &stream);
+
        void set_header(const std::string &stream_id, const std::string &header);
        void add_data(const std::string &stream_id, const char *data, size_t bytes);
 
@@ -79,7 +108,7 @@ private:
        bool should_stop;       
 
        // Map from stream ID to stream.
-       std::map<std::string, Stream> streams;
+       std::map<std::string, Stream *> streams;
 
        // Map from file descriptor to client.
        std::map<int, Client> clients;
@@ -104,9 +133,16 @@ private:
        // Close a given client socket, and clean up after it.
        void close_client(Client *client);
 
-       // Parse the HTTP request, construct the header, and set the client into
+       // Parse the HTTP request. Returns a HTTP status code (200/400/404).
+       int parse_request(Client *client);
+
+       // Construct the HTTP header, and set the client into
        // the SENDING_HEADER state.
-       void parse_request(Client *client);
+       void construct_header(Client *client);
+
+       // Construct a generic error with the given line, and set the client into
+       // the SENDING_ERROR state.
+       void construct_error(Client *client, int error_code);
 
        // Put client to sleep, since there is no more data for it; we will on
        // longer listen on POLLOUT until we get more data. Also, it will be put
@@ -115,6 +151,9 @@ private:
 
        // We have more data, so mark all clients that are sleeping as ready to go.
        void wake_up_all_clients();
+
+       // TODO: This function should probably die.
+       Stream *find_stream(const std::string &stream_id);
 };
 
 #endif  // !defined(_SERVER_H)